﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using UnitOfWorkDemo.Infrastructure;
using UnitOfWorkDemo.Model;
using UnitOfWorkDemo.Repository;

namespace UnitOfWorkDemo.Test
{
    [TestClass]
    public class AccountRepositoryTests
    {
        [TestMethod]
        public void AccountRepository_Delegates_Changes_To_The_Unit_Of_Work_Instance()
        {
            var accountToBeAmended = new Account();
            var accountToBeRemoved = new Account();
            var accountToBeAdded = new Account();

            // 需要引入Moq Mock框架
            var unitOfWorkMockery = new Mock<IUnitOfWork>();

            var accountRepository = new AccountRepository(unitOfWorkMockery.Object);

            unitOfWorkMockery.Setup(uow => uow.RegisterAmended(accountToBeAmended, accountRepository));
            unitOfWorkMockery.Setup(uow => uow.RegisterNew(accountToBeAdded, accountRepository));
            unitOfWorkMockery.Setup(uow => uow.RegisterRemoved(accountToBeRemoved, accountRepository));

            accountRepository.Add(accountToBeAdded);
            accountRepository.Save(accountToBeAmended);
            accountRepository.Remove(accountToBeRemoved);

            unitOfWorkMockery.VerifyAll();
        }        
    }
}
