﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using UnitOfWorkDemo.Infrastructure;
using UnitOfWorkDemo.Model;

namespace UnitOfWorkDemo.Test
{
    [TestClass]
    public class UnitOfWorkTests
    {
        [TestMethod]
        public void UnitOfWorkRepository_Is_Used_For_The_Real_Persistence_When_Commit_Is_Called_On_The_Unit_Of_Work()
        {
            var accountToBeAmended = new Account();
            var accountToBeRemoved = new Account();
            var accountToBeAdded = new Account();

            var uow = new Infrastructure.UnitOfWork();

            // 需要引入Moq Mock框架
            var uowRepositoryMockery = new Mock<IUnitOfWorkRepository>();

            var uowRepository = uowRepositoryMockery.Object;

            uowRepositoryMockery.Setup(uowr => uowr.PersistCreationOf(accountToBeAdded));
            uowRepositoryMockery.Setup(uowr => uowr.PersistDeletionOf(accountToBeRemoved));
            uowRepositoryMockery.Setup(uowr => uowr.PersistUpdateOf(accountToBeAmended));

            uow.RegisterNew(accountToBeAdded, uowRepository);
            uow.RegisterAmended(accountToBeAmended, uowRepository);
            uow.RegisterRemoved(accountToBeRemoved, uowRepository);

            uow.Commit();

            uowRepositoryMockery.VerifyAll();
        } 
    }
}